==> galois-0.1.tar.gz: galois-0.1/src/main.cc <==
/* main.cc -- main window for falling blocks game.  -*- C++ -*-
   Copyright (C) 2011-2012 Gerardo Ballabio

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <sstream>
#include <vector>
#include <gtkmm.h>
#include "board.h"
#include "config.h"
#include "cube.h"
#include "group.h"
#include "hexagon.h"
#include "main.h"
#include "preferences.h"
#include "square.h"
#include "triangle.h"

bool MainWindow::on_key_press_event(GdkEventKey *event)
{
  if (board.is_game())
    {
      unsigned key = event->keyval;
      if (pause)
	{
	  if (key == (board.get_group()->is_3d() ?
		      preferences.get_key_3d(9) : preferences.get_key_2d(5)))
	    {
	      pause = false;
	      board_area.set_pause(false);

	      // resume game
	      timeout.disconnect();
	      delay = 1000 / board.get_speed();
	      timeout = Glib::signal_timeout().connect
		(sigc::mem_fun(*this, &MainWindow::on_timeout_down), delay);
	    }
	}
      else
	{
	  bool refresh = false;
	  if (board.get_group()->is_3d())
	    {
	      // move in four directions, three rotations, one reflection
	      if (key == preferences.get_key_3d(0))
		refresh = board.left();
	      else if (key == preferences.get_key_3d(1))
		refresh = board.right();
	      else if (key == preferences.get_key_3d(2))
		refresh = board.back();
	      else if (key == preferences.get_key_3d(3))
		refresh = board.front();
	      else if (key == preferences.get_key_3d(4))
		refresh = board.rotate(0);
	      else if (key == preferences.get_key_3d(5))
		refresh = board.rotate(1);
	      else if (key == preferences.get_key_3d(6))
		refresh = board.rotate(2);
	      else if (key == preferences.get_key_3d(7))
		refresh = board.reflect();
	      else if (key == preferences.get_key_3d(8))
		{
		  board.drop(); // don't refresh now, on_game_next takes care
		  on_game_next();
		}
	      else if (key == preferences.get_key_3d(9))
		{
		  pause = true;
		  board_area.set_pause(true);
		  board_area.set_pause_key(gdk_keyval_name(key));
		}
	    }
	  else
	    {
	      // move in two directions, one rotation, one reflection
	      if (key == preferences.get_key_2d(0))
		refresh = board.left();
	      else if (key == preferences.get_key_2d(1))
		refresh = board.right();
	      else if (key == preferences.get_key_2d(2))
		refresh = board.rotate(0);
	      else if (key == preferences.get_key_2d(3))
		refresh = board.reflect();
	      else if (key == preferences.get_key_2d(4))
		{
		  board.drop(); // don't refresh now, on_game_next takes care
		  on_game_next();
		}
	      else if (key == preferences.get_key_2d(5))
		{
		  pause = true;
		  board_area.set_pause(true);
		  board_area.set_pause_key(gdk_keyval_name(key));
		}
	    }

	  if (refresh)
	    board_area.set_block(board.get_block(),
				 land ? board.get_shadow() : grid<bool>(),
				 board.get_iblock());
	}
    }

  Gtk::Window::on_key_press_event(event); // default handler
  return true;
}

void MainWindow::on_game_new()
{
  if (board.is_game())
    return;

  // update actions status
  actiongroup->get_action("GameNew")->set_sensitive(false);
  actiongroup->get_action("GameStop")->set_sensitive(true);
  actiongroup->get_action("GamePrefs")->set_sensitive(false);
  actiongroup->get_action("GameScores")->set_sensitive(false);

  // select spatial group
  int geometry = preferences.get_geometry();
  bool reflection = preferences.get_reflection();
  const group *g;
  switch (geometry)
    {
    case 0:
      if (reflection)
	g = new group_square_mirror;
      else
	g = new group_square;
      break;
    case 1:
      if (reflection)
	g = new group_hexagon_mirror;
      else
	g = new group_hexagon;
      break;
    case 2:
      if (reflection)
	g = new group_triangle_mirror;
      else
	g = new group_triangle;
      break;
    case 3:
      if (reflection)
	g = new group_cube_mirror;
      else
	g = new group_cube;
      break;
    }
  // board width
  int width = preferences.get_width(), depth = preferences.get_depth();

  // level change mode
  int mode1 = preferences.get_mode1(), mode2 = preferences.get_mode2();

  // speed level
  int speed = preferences.get_speed();

  // number of bricks per block
  int bmax = preferences.get_bricks_max(),
    bmin = preferences.get_bricks_min();

  // preview next block
  next = preferences.get_next();

  // show where block will land
  land = preferences.get_land();

  // empty cells per line
  int empty = preferences.get_empty();

  // start game
  animation = 0;
  pause = false;
  board.start_game(g, width, depth, mode1, mode2, speed, bmin, bmax, empty);
  board_area.start_game(g, board.get_cells(), board.get_block(),
			board.get_iblock());
  next_area.set_group(g);
  if (next)
    next_area.set_block(board.get_next_block(), board.get_nblock());
  else
    next_area.set_block(grid<bool>(), 0);

  // game statistics
  std::ostringstream o;
  o << "Level: " << board.get_level();
  level_label.set_text(o.str());
  lines_label.set_text("Lines: 0");
  score_label.set_text("Score: 0");

  // start timeout
  delay = 1000 / board.get_speed();
  timeout = Glib::signal_timeout().connect
    (sigc::mem_fun(*this, &MainWindow::on_timeout_down), delay);
}

void MainWindow::on_game_stop()
{
  if (!board.is_game())
    return;

  board.stop();
  board_area.set_block(grid<bool>(), grid<bool>(), 0);
  next_area.set_block(grid<bool>(), 0);

  // stop timeout
  timeout.disconnect();

  // update actions status
  actiongroup->get_action("GameNew")->set_sensitive(true);
  actiongroup->get_action("GameStop")->set_sensitive(false);
  actiongroup->get_action("GamePrefs")->set_sensitive(true);
  actiongroup->get_action("GameScores")->set_sensitive(true);
}

void MainWindow::on_game_prefs()
{
  preferences.reset_page();
  preferences.show();
}

void MainWindow::on_game_scores()
{
  // I would place this in the constructor
  // if I could made sure that scores is initialized after preferences
  static bool first = true;
  if (first)
    {
      first = false;
      scores.filter(preferences.get_geometry(), preferences.get_reflection(),
		    preferences.get_width(), preferences.get_depth(),
		    preferences.get_mode1(), preferences.get_mode2(),
		    preferences.get_speed(), preferences.get_bricks_max(),
		    preferences.get_bricks_min(), preferences.get_next(),
		    preferences.get_land(), preferences.get_empty());
    }

  scores.show();
}

void MainWindow::on_help_contents()
{
  // use system until I figure out how to do it properly
  std::system("yelp " HELP_FILE " &");
}

void MainWindow::on_help_about()
{
  Gtk::AboutDialog dialog;
  dialog.set_name("Galois");
  dialog.set_version("0.1");
  dialog.set_comments("A falling blocks game with many different geometries");
  dialog.set_license
    ("Galois is free software; you can redistribute it and/or modify\n"
     "it under the terms of the GNU General Public License as published by\n"
     "the Free Software Foundation; either version 3 of the License, or\n"
     "(at your option) any later version.\n"
     "\n"
     "Galois is distributed in the hope that it will be useful, but\n"
     "WITHOUT ANY WARRANTY; without even the implied warranty of\n"
     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
     "GNU General Public License for more details.\n"
     "\n"
     "You should have received a copy of the GNU General Public License\n"
     "along with Galois.  If not, see <http://www.gnu.org/licenses/>.");
  dialog.run();
}

void MainWindow::on_response_prefs(int response_id)
{
  if (response_id == Gtk::RESPONSE_DELETE_EVENT)
    preferences.hide();
}

void MainWindow::on_response_scores(int response_id)
{
  if (response_id == Gtk::RESPONSE_DELETE_EVENT)
    scores.hide();
}

bool MainWindow::on_timeout_down()
{
  if (pause)
    timeout.disconnect();
  else if (!board.down())
    on_game_next();
  else
    board_area.set_block(board.get_block(),
			 land ? board.get_shadow() : grid<bool>(),
			 board.get_iblock());

  return true;
}

bool MainWindow::on_timeout_lines()
{
  if (animation > 0)
    {
      board.color_lines(--animation);
      board_area.queue_draw();
      return true;
    }
  // else

  board.remove_lines();

  // update scores
  std::ostringstream o1, o2, o3;
  o1 << "Level: " << board.get_level();
  level_label.set_text(o1.str());
  o2 << "Lines: " << board.get_lines();
  lines_label.set_text(o2.str());
  o3 << "Score: " << board.get_score();
  score_label.set_text(o3.str());

  // continue game
  timeout.disconnect();
  delay = 1000 / board.get_speed();
  timeout = Glib::signal_timeout().connect
    (sigc::mem_fun(*this, &MainWindow::on_timeout_down), delay);

  // new blocks
  board_area.set_block(board.get_block(),
		       land ? board.get_shadow() : grid<bool>(),
		       board.get_iblock());
  if (next)
    next_area.set_block(board.get_next_block(), board.get_nblock());

  // check for end of game
  if (!board.is_game())
    on_game_over();

  return true;
}

void MainWindow::on_game_next()
{
  // update score
  std::ostringstream o;
  o << "Score: " << board.get_score();
  score_label.set_text(o.str());

  std::vector<int> filled = board.get_filled();
  if (filled.size() > 0)
    {
      // switch timeout
      timeout.disconnect();
      timeout = Glib::signal_timeout().connect
	(sigc::mem_fun(*this, &MainWindow::on_timeout_lines), 20);
      animation = ncolors;

      // refresh, but without active block
      board_area.set_block(grid<bool>(), grid<bool>(), 0);
    }
  else
    {
      // new blocks
      board.next_block();
      board_area.set_block(board.get_block(),
			   land ? board.get_shadow() : grid<bool>(),
			   board.get_iblock());
      if (next)
	next_area.set_block(board.get_next_block(), board.get_nblock());

      // check for end of game
      if (!board.is_game())
	on_game_over();
    }
}

void MainWindow::on_game_over()
{
  next_area.set_block(grid<bool>(), 0);

  // stop timeout
  timeout.disconnect();

  // register score
  Glib::ustring name = Glib::get_real_name();
  if (name == "")
    name = Glib::get_user_name();
  Glib::TimeVal timeval;
  timeval.assign_current_time();
  double time = timeval.as_double();
  scores.add(name, time,
	     board.get_score(), board.get_lines(), board.get_level(),
	     board.get_speed(), board.get_bmax(), board.get_bmin(),
	     preferences.get_geometry(), preferences.get_reflection(),
	     preferences.get_width(), preferences.get_depth(),
	     preferences.get_mode1(), preferences.get_mode2(),
	     preferences.get_speed(),
	     preferences.get_bricks_max(), preferences.get_bricks_min(),
	     preferences.get_next(), preferences.get_land(),
	     preferences.get_empty());
  if (scores.is_active())
    scores.show();

  // update actions status
  actiongroup->get_action("GameNew")->set_sensitive(true);
  actiongroup->get_action("GameStop")->set_sensitive(false);
  actiongroup->get_action("GamePrefs")->set_sensitive(true);
  actiongroup->get_action("GameScores")->set_sensitive(true);
}

MainWindow::MainWindow()
  : level_label("", 0.0, 0.5), lines_label("", 0.0, 0.5),
    score_label("", 0.0, 0.5)
{
  add(vbox_all);

  // menu bar and tool bar
  actiongroup = Gtk::ActionGroup::create();
  actiongroup->add(Gtk::Action::create("GameMenu", "_Game"));
  actiongroup->add(Gtk::Action::create("GameNew", Gtk::Stock::NEW),
		   sigc::mem_fun(*this, &MainWindow::on_game_new));
  actiongroup->add(Gtk::Action::create("GameStop", Gtk::Stock::STOP),
		   sigc::mem_fun(*this, &MainWindow::on_game_stop));
  actiongroup->add(Gtk::Action::create("GamePrefs", Gtk::Stock::PREFERENCES),
		   sigc::mem_fun(*this, &MainWindow::on_game_prefs));
  actiongroup->add(Gtk::Action::create("GameScores", "_Scores"),
		   sigc::mem_fun(*this, &MainWindow::on_game_scores));
  actiongroup->add(Gtk::Action::create("GameQuit", Gtk::Stock::QUIT),
		   sigc::mem_fun(*this, &MainWindow::on_game_quit));
  actiongroup->add(Gtk::Action::create("HelpMenu", "_Help"));
  actiongroup->add(Gtk::Action::create("HelpContents", Gtk::Stock::HELP,
				       "_Contents"), Gtk::AccelKey("F1"),
		   sigc::mem_fun(*this, &MainWindow::on_help_contents));
  actiongroup->add(Gtk::Action::create("HelpAbout", Gtk::Stock::ABOUT),
		   sigc::mem_fun(*this, &MainWindow::on_help_about));
  uimanager = Gtk::UIManager::create();
  uimanager->insert_action_group(actiongroup);
  Glib::ustring ui_info =
    "<ui>"
    "  <menubar name='MenuBar'>"
    "    <menu action='GameMenu'>"
    "      <menuitem action='GameNew'/>"
    "      <menuitem action='GameStop'/>"
    "      <menuitem action='GamePrefs'/>"
    "      <menuitem action='GameScores'/>"
    "      <menuitem action='GameQuit'/>"
    "    </menu>"
    "    <menu action='HelpMenu'>"
    "      <menuitem action='HelpContents'/>"
    "      <menuitem action='HelpAbout'/>"
    "    </menu>"
    "  </menubar>"
    "  <toolbar name='ToolBar'>"
    "    <toolitem action='GameNew'/>"
    "  </toolbar>"
    "</ui>";
  uimanager->add_ui_from_string(ui_info);
  add_accel_group(uimanager->get_accel_group());
  vbox_all.pack_start(vbox_ui, Gtk::PACK_SHRINK);
  Gtk::Widget *menubar = uimanager->get_widget("/MenuBar");
  vbox_ui.pack_start(*menubar, Gtk::PACK_SHRINK);
  // Gtk::Widget *toolbar = uimanager->get_widget("/ToolBar");
  // vbox_ui.pack_start(*toolbar, Gtk::PACK_SHRINK);
  actiongroup->get_action("GameStop")->set_sensitive(false);

  vbox_all.pack_start(hbox, Gtk::PACK_EXPAND_WIDGET);
  hbox.pack_start(board_area, Gtk::PACK_EXPAND_WIDGET);
  hbox.pack_start(vbox, Gtk::PACK_EXPAND_WIDGET);
  vbox.pack_start(next_area, Gtk::PACK_EXPAND_WIDGET);
  next_area.set_board_area(&board_area);
  vbox.pack_start(vbox2, Gtk::PACK_EXPAND_WIDGET);
  vbox2.pack_start(level_label, Gtk::PACK_SHRINK);
  vbox2.pack_start(lines_label, Gtk::PACK_SHRINK);
  vbox2.pack_start(score_label, Gtk::PACK_SHRINK);

  // connect signals
  preferences.signal_response().connect
    (sigc::mem_fun(*this, &MainWindow::on_response_prefs));
  scores.signal_response().connect
    (sigc::mem_fun(*this, &MainWindow::on_response_scores));

  show_all_children();
}

int main(int argc, char *argv[])
{
  // initialize gtkmm
  Gtk::Main kit(argc, argv);

  // create main window
  MainWindow window;

  // main loop
  Gtk::Main::run(window);

  return 0;
}

==> galois_0.2.orig.tar.gz: galois-0.2/src/main.cc <==
/* main.cc -- main window for falling blocks game.  -*- C++ -*-
   Copyright (C) 2011-2013 Gerardo Ballabio

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <sstream>
#include <vector>
#include <gtkmm.h>
#include "board.h"
#include "cube.h"
#include "group.h"
#include "hexagon.h"
#include "main.h"
#include "preferences.h"
#include "square.h"
#include "triangle.h"

bool MainWindow::on_key_press_event(GdkEventKey *event)
{
  if (board.is_game())
    {
      unsigned key = event->keyval;
      if (pause)
	{
	  if (key == (board.get_group()->is_3d() ?
		      preferences.get_key_3d(9) : preferences.get_key_2d(5)))
	    {
	      pause = false;
	      board_area.set_pause(false);

	      // resume game
	      timeout.disconnect();
	      delay = 1000 / board.get_speed();
	      timeout = Glib::signal_timeout().connect
		(sigc::mem_fun(*this, &MainWindow::on_timeout_down), delay);
	    }
	}
      else
	{
	  bool refresh = false;
	  if (board.get_group()->is_3d())
	    {
	      // move in four directions, three rotations, one reflection
	      if (key == preferences.get_key_3d(0))
		refresh = board.left();
	      else if (key == preferences.get_key_3d(1))
		refresh = board.right();
	      else if (key == preferences.get_key_3d(2))
		refresh = board.back();
	      else if (key == preferences.get_key_3d(3))
		refresh = board.front();
	      else if (key == preferences.get_key_3d(4))
		refresh = board.rotate(0);
	      else if (key == preferences.get_key_3d(5))
		refresh = board.rotate(1);
	      else if (key == preferences.get_key_3d(6))
		refresh = board.rotate(2);
	      else if (key == preferences.get_key_3d(7))
		refresh = board.reflect();
	      else if (key == preferences.get_key_3d(8))
		{
		  board.drop(); // don't refresh now, on_game_next takes care
		  on_game_next();
		}
	      else if (key == preferences.get_key_3d(9))
		{
		  pause = true;
		  board_area.set_pause(true);
		  board_area.set_pause_key(gdk_keyval_name(key));
		}
	    }
	  else
	    {
	      // move in two directions, one rotation, one reflection
	      if (key == preferences.get_key_2d(0))
		refresh = board.left();
	      else if (key == preferences.get_key_2d(1))
		refresh = board.right();
	      else if (key == preferences.get_key_2d(2))
		refresh = board.rotate(0);
	      else if (key == preferences.get_key_2d(3))
		refresh = board.reflect();
	      else if (key == preferences.get_key_2d(4))
		{
		  board.drop(); // don't refresh now, on_game_next takes care
		  on_game_next();
		}
	      else if (key == preferences.get_key_2d(5))
		{
		  pause = true;
		  board_area.set_pause(true);
		  board_area.set_pause_key(gdk_keyval_name(key));
		}
	    }

	  if (refresh)
	    board_area.set_block(board.get_block(),
				 land ? board.get_shadow() : grid<bool>(),
				 board.get_iblock());
	}
    }

  Gtk::Window::on_key_press_event(event); // default handler
  return true;
}

void MainWindow::on_game_new()
{
  if (board.is_game())
    return;

  // update actions status
  actiongroup->get_action("GameNew")->set_sensitive(false);
  actiongroup->get_action("GameStop")->set_sensitive(true);
  actiongroup->get_action("GamePrefs")->set_sensitive(false);
  actiongroup->get_action("GameScores")->set_sensitive(false);

  // select spatial group
  int geometry = preferences.get_geometry();
  bool reflection = preferences.get_reflection();
  const group *g = 0;
  switch (geometry)
    {
    case 0:
      if (reflection)
	g = new group_square_mirror;
      else
	g = new group_square;
      break;
    case 1:
      if (reflection)
	g = new group_hexagon_mirror;
      else
	g = new group_hexagon;
      break;
    case 2:
      if (reflection)
	g = new group_triangle_mirror;
      else
	g = new group_triangle;
      break;
    case 3:
      if (reflection)
	g = new group_cube_mirror;
      else
	g = new group_cube;
      break;
    }

  // board width
  int width = preferences.get_width(), depth = preferences.get_depth();

  // level change mode
  int mode1 = preferences.get_mode1(), mode2 = preferences.get_mode2();

  // speed level
  int speed = preferences.get_speed();

  // number of bricks per block
  int bmax = preferences.get_bricks_max(),
    bmin = preferences.get_bricks_min();

  // preview next block
  next = preferences.get_next();

  // show where block will land
  land = preferences.get_land();

  // empty cells per line
  int empty = preferences.get_empty();

  // start game
  animation = 0;
  pause = false;
  board.start_game(g, width, depth, mode1, mode2, speed, bmin, bmax, empty);
  board_area.start_game(g, board.get_cells(), board.get_block(),
			board.get_iblock());
  next_area.set_group(g);
  if (next)
    next_area.set_block(board.get_next_block(), board.get_nblock());
  else
    next_area.set_block(grid<bool>(), 0);

  // game statistics
  std::ostringstream o;
  o << "Level: " << board.get_level();
  level_label.set_text(o.str());
  lines_label.set_text("Lines: 0");
  score_label.set_text("Score: 0");

  // start timeout
  delay = 1000 / board.get_speed();
  timeout = Glib::signal_timeout().connect
    (sigc::mem_fun(*this, &MainWindow::on_timeout_down), delay);
}

void MainWindow::on_game_stop()
{
  if (!board.is_game())
    return;

  board.stop();
  board_area.set_pause(false);
  // set y to negative so that plane isn't drawn in 3D game
  board_area.set_block(grid<bool>(0, 0, 0, 0, -1, 0), grid<bool>(), 0);
  next_area.set_block(grid<bool>(), 0);

  // stop timeout
  timeout.disconnect();

  // update actions status
  actiongroup->get_action("GameNew")->set_sensitive(true);
  actiongroup->get_action("GameStop")->set_sensitive(false);
  actiongroup->get_action("GamePrefs")->set_sensitive(true);
  actiongroup->get_action("GameScores")->set_sensitive(true);
}

void MainWindow::on_game_prefs()
{
  preferences.reset_page();
  preferences.show();
}

void MainWindow::on_game_scores()
{
  // I would place this in the constructor
  // if I could made sure that scores is initialized after preferences
  static bool first = true;
  if (first)
    {
      first = false;
      scores.filter(preferences.get_geometry(), preferences.get_reflection(),
		    preferences.get_width(), preferences.get_depth(),
		    preferences.get_mode1(), preferences.get_mode2(),
		    preferences.get_speed(), preferences.get_bricks_max(),
		    preferences.get_bricks_min(), preferences.get_next(),
		    preferences.get_land(), preferences.get_empty());
    }

  scores.show();
}

void MainWindow::on_help_contents()
{
  // use system until I figure out how to do it properly
  std::system("yelp ghelp:galois &");
}

void MainWindow::on_help_about()
{
  Gtk::AboutDialog dialog;
  dialog.set_name("Galois");
  dialog.set_version("0.2");
  dialog.set_comments("A falling blocks game with many different geometries");
  dialog.set_license
    ("Galois is free software; you can redistribute it and/or modify\n"
     "it under the terms of the GNU General Public License as published by\n"
     "the Free Software Foundation; either version 3 of the License, or\n"
     "(at your option) any later version.\n"
     "\n"
     "Galois is distributed in the hope that it will be useful, but\n"
     "WITHOUT ANY WARRANTY; without even the implied warranty of\n"
     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
     "GNU General Public License for more details.\n"
     "\n"
     "You should have received a copy of the GNU General Public License\n"
     "along with Galois.  If not, see <http://www.gnu.org/licenses/>.");
  dialog.run();
}

void MainWindow::on_response_prefs(int response_id)
{
  if (response_id == Gtk::RESPONSE_DELETE_EVENT)
    preferences.hide();
}

void MainWindow::on_response_scores(int response_id)
{
  if (response_id == Gtk::RESPONSE_DELETE_EVENT)
    scores.hide();
}

bool MainWindow::on_timeout_down()
{
  if (pause)
    timeout.disconnect();
  else if (!board.down())
    on_game_next();
  else
    board_area.set_block(board.get_block(),
			 land ? board.get_shadow() : grid<bool>(),
			 board.get_iblock());

  return true;
}

bool MainWindow::on_timeout_lines()
{
  if (animation > 0)
    {
      board.color_lines(--animation);
      board_area.queue_draw();
      return true;
    }
  // else

  board.remove_lines();

  // update scores
  std::ostringstream o1, o2, o3;
  o1 << "Level: " << board.get_level();
  level_label.set_text(o1.str());
  o2 << "Lines: " << board.get_lines();
  lines_label.set_text(o2.str());
  o3 << "Score: " << board.get_score();
  score_label.set_text(o3.str());

  // continue game
  timeout.disconnect();
  delay = 1000 / board.get_speed();
  timeout = Glib::signal_timeout().connect
    (sigc::mem_fun(*this, &MainWindow::on_timeout_down), delay);

  // new blocks
  board_area.set_block(board.get_block(),
		       land ? board.get_shadow() : grid<bool>(),
		       board.get_iblock());
  if (next)
    next_area.set_block(board.get_next_block(), board.get_nblock());

  // check for end of game
  if (!board.is_game())
    on_game_over();

  return true;
}

void MainWindow::on_game_next()
{
  // update score
  std::ostringstream o;
  o << "Score: " << board.get_score();
  score_label.set_text(o.str());

  std::vector<int> filled = board.get_filled();
  if (filled.size() > 0)
    {
      // switch timeout
      timeout.disconnect();
      timeout = Glib::signal_timeout().connect
	(sigc::mem_fun(*this, &MainWindow::on_timeout_lines), 20);
      animation = ncolors;

      // refresh, but without active block
      board_area.set_block(grid<bool>(), grid<bool>(), 0);
    }
  else
    {
      // new blocks
      board.next_block();
      board_area.set_block(board.get_block(),
			   land ? board.get_shadow() : grid<bool>(),
			   board.get_iblock());
      if (next)
	next_area.set_block(board.get_next_block(), board.get_nblock());

      // check for end of game
      if (!board.is_game())
	on_game_over();
    }
}

void MainWindow::on_game_over()
{
  next_area.set_block(grid<bool>(), 0);

  // stop timeout
  timeout.disconnect();

  // register score
  Glib::ustring name = Glib::get_real_name();
  if (name == "")
    name = Glib::get_user_name();
  Glib::TimeVal timeval;
  timeval.assign_current_time();
  double time = timeval.as_double();
  scores.add(name, time,
	     board.get_score(), board.get_lines(), board.get_level(),
	     board.get_speed(), board.get_bmax(), board.get_bmin(),
	     preferences.get_geometry(), preferences.get_reflection(),
	     preferences.get_width(), preferences.get_depth(),
	     preferences.get_mode1(), preferences.get_mode2(),
	     preferences.get_speed(),
	     preferences.get_bricks_max(), preferences.get_bricks_min(),
	     preferences.get_next(), preferences.get_land(),
	     preferences.get_empty());
  if (scores.is_active())
    scores.show();

  // update actions status
  actiongroup->get_action("GameNew")->set_sensitive(true);
  actiongroup->get_action("GameStop")->set_sensitive(false);
  actiongroup->get_action("GamePrefs")->set_sensitive(true);
  actiongroup->get_action("GameScores")->set_sensitive(true);
}

MainWindow::MainWindow()
  : level_label("", 0.0, 0.5), lines_label("", 0.0, 0.5),
    score_label("", 0.0, 0.5)
{
  add(vbox_all);

  // menu bar and tool bar
  actiongroup = Gtk::ActionGroup::create();
  actiongroup->add(Gtk::Action::create("GameMenu", "_Game"));
  actiongroup->add(Gtk::Action::create("GameNew", Gtk::Stock::NEW),
		   sigc::mem_fun(*this, &MainWindow::on_game_new));
  actiongroup->add(Gtk::Action::create("GameStop", Gtk::Stock::STOP),
		   sigc::mem_fun(*this, &MainWindow::on_game_stop));
  actiongroup->add(Gtk::Action::create("GamePrefs", Gtk::Stock::PREFERENCES),
		   sigc::mem_fun(*this, &MainWindow::on_game_prefs));
  actiongroup->add(Gtk::Action::create("GameScores", "_Scores"),
		   sigc::mem_fun(*this, &MainWindow::on_game_scores));
  actiongroup->add(Gtk::Action::create("GameQuit", Gtk::Stock::QUIT),
		   sigc::mem_fun(*this, &MainWindow::on_game_quit));
  actiongroup->add(Gtk::Action::create("HelpMenu", "_Help"));
  actiongroup->add(Gtk::Action::create("HelpContents", Gtk::Stock::HELP,
				       "_Contents"), Gtk::AccelKey("F1"),
		   sigc::mem_fun(*this, &MainWindow::on_help_contents));
  actiongroup->add(Gtk::Action::create("HelpAbout", Gtk::Stock::ABOUT),
		   sigc::mem_fun(*this, &MainWindow::on_help_about));
  uimanager = Gtk::UIManager::create();
  uimanager->insert_action_group(actiongroup);
  Glib::ustring ui_info =
    "<ui>"
    "  <menubar name='MenuBar'>"
    "    <menu action='GameMenu'>"
    "      <menuitem action='GameNew'/>"
    "      <menuitem action='GameStop'/>"
    "      <menuitem action='GamePrefs'/>"
    "      <menuitem action='GameScores'/>"
    "      <menuitem action='GameQuit'/>"
    "    </menu>"
    "    <menu action='HelpMenu'>"
    "      <menuitem action='HelpContents'/>"
    "      <menuitem action='HelpAbout'/>"
    "    </menu>"
    "  </menubar>"
    "  <toolbar name='ToolBar'>"
    "    <toolitem action='GameNew'/>"
    "  </toolbar>"
    "</ui>";
  uimanager->add_ui_from_string(ui_info);
  add_accel_group(uimanager->get_accel_group());
  vbox_all.pack_start(vbox_ui, Gtk::PACK_SHRINK);
  Gtk::Widget *menubar = uimanager->get_widget("/MenuBar");
  vbox_ui.pack_start(*menubar, Gtk::PACK_SHRINK);
  // Gtk::Widget *toolbar = uimanager->get_widget("/ToolBar");
  // vbox_ui.pack_start(*toolbar, Gtk::PACK_SHRINK);
  actiongroup->get_action("GameStop")->set_sensitive(false);

  vbox_all.pack_start(hbox, Gtk::PACK_EXPAND_WIDGET);
  hbox.pack_start(board_area, Gtk::PACK_EXPAND_WIDGET);
  hbox.pack_start(vbox, Gtk::PACK_EXPAND_WIDGET);
  vbox.pack_start(next_area, Gtk::PACK_EXPAND_WIDGET);
  next_area.set_board_area(&board_area);
  vbox.pack_start(vbox2, Gtk::PACK_EXPAND_WIDGET);
  vbox2.pack_start(level_label, Gtk::PACK_SHRINK);
  vbox2.pack_start(lines_label, Gtk::PACK_SHRINK);
  vbox2.pack_start(score_label, Gtk::PACK_SHRINK);

  // connect signals
  preferences.signal_response().connect
    (sigc::mem_fun(*this, &MainWindow::on_response_prefs));
  scores.signal_response().connect
    (sigc::mem_fun(*this, &MainWindow::on_response_scores));

  show_all_children();
}

int main(int argc, char *argv[])
{
  // initialize gtkmm
  Gtk::Main kit(argc, argv);

  // create main window
  MainWindow window;

  // main loop
  Gtk::Main::run(window);

  return 0;
}
